import { __read, __spread } from 'tslib';
import { Injectable, Inject, PLATFORM_ID, NgModule } from '@angular/core';
import { HttpBackend, HttpRequest, HttpResponse, HttpHeaders, HttpParams } from '@angular/common/http';
import { filter, map, catchError, retry, switchMap } from 'rxjs/operators';
import { decode } from 'vlq';
import { of } from 'rxjs';
import { isPlatformBrowser, CommonModule } from '@angular/common';

var NGXLoggerHttpService = /** @class */ (function () {
    function NGXLoggerHttpService(httpBackend) {
        this.httpBackend = httpBackend;
    }
    NGXLoggerHttpService.prototype.logOnServer = function (url, log, options) {
        var req = new HttpRequest('POST', url, log, options || {});
        return this.httpBackend.handle(req).pipe(filter(function (e) { return e instanceof HttpResponse; }), map(function (httpResponse) { return httpResponse.body; }));
    };
    return NGXLoggerHttpService;
}());
NGXLoggerHttpService.decorators = [
    { type: Injectable },
];
NGXLoggerHttpService.ctorParameters = function () { return [
    { type: HttpBackend }
]; };
var NgxLoggerLevel = {
    TRACE: 0,
    DEBUG: 1,
    INFO: 2,
    LOG: 3,
    WARN: 4,
    ERROR: 5,
    FATAL: 6,
    OFF: 7,
};
NgxLoggerLevel[NgxLoggerLevel.TRACE] = 'TRACE';
NgxLoggerLevel[NgxLoggerLevel.DEBUG] = 'DEBUG';
NgxLoggerLevel[NgxLoggerLevel.INFO] = 'INFO';
NgxLoggerLevel[NgxLoggerLevel.LOG] = 'LOG';
NgxLoggerLevel[NgxLoggerLevel.WARN] = 'WARN';
NgxLoggerLevel[NgxLoggerLevel.ERROR] = 'ERROR';
NgxLoggerLevel[NgxLoggerLevel.FATAL] = 'FATAL';
NgxLoggerLevel[NgxLoggerLevel.OFF] = 'OFF';
var LoggerConfig = /** @class */ (function () {
    function LoggerConfig() {
    }
    return LoggerConfig;
}());
var NGXLoggerConfigEngine = /** @class */ (function () {
    function NGXLoggerConfigEngine(config) {
        this.config = config;
        this._config = config;
    }
    NGXLoggerConfigEngine.prototype.updateConfig = function (config) {
        this._config = this._clone(config);
    };
    NGXLoggerConfigEngine.prototype.getConfig = function () {
        return this._clone(this._config);
    };
    NGXLoggerConfigEngine.prototype._clone = function (object) {
        var cloneConfig = new LoggerConfig();
        Object.keys(object).forEach(function (key) {
            cloneConfig[key] = object[key];
        });
        return cloneConfig;
    };
    return NGXLoggerConfigEngine;
}());
var NGXLoggerUtils = /** @class */ (function () {
    function NGXLoggerUtils() {
    }
    NGXLoggerUtils.prepareMetaString = function (timestamp, logLevel, fileName, lineNumber) {
        var fileDetails = fileName ? " [" + fileName + ":" + lineNumber + "]" : '';
        return timestamp + " " + logLevel + fileDetails;
    };
    NGXLoggerUtils.getColor = function (level) {
        switch (level) {
            case NgxLoggerLevel.TRACE:
                return 'blue';
            case NgxLoggerLevel.DEBUG:
                return 'teal';
            case NgxLoggerLevel.INFO:
            case NgxLoggerLevel.LOG:
                return 'gray';
            case NgxLoggerLevel.WARN:
            case NgxLoggerLevel.ERROR:
            case NgxLoggerLevel.FATAL:
                return 'red';
            case NgxLoggerLevel.OFF:
            default:
                return;
        }
    };
    NGXLoggerUtils.getCallerDetails = function () {
        var err = (new Error(''));
        try {
            var callerLine = err.stack.split('\n')[4].split('/');
            var fileLineNumber = callerLine[callerLine.length - 1].replace(/[)]/g, '').split(':');
            return {
                fileName: fileLineNumber[0],
                lineNumber: fileLineNumber[1]
            };
        }
        catch (e) {
            return {
                fileName: null,
                lineNumber: null
            };
        }
    };
    NGXLoggerUtils.prepareMessage = function (message) {
        try {
            if (typeof message !== 'string' && !(message instanceof Error)) {
                message = JSON.stringify(message, null, 2);
            }
        }
        catch (e) {
            message = 'The provided "message" value could not be parsed with JSON.stringify().';
        }
        return message;
    };
    NGXLoggerUtils.prepareAdditionalParameters = function (additional) {
        if (additional === null || additional === undefined) {
            return null;
        }
        return additional.map(function (next, idx) {
            try {
                if (typeof next === 'object') {
                    JSON.stringify(next);
                }
                return next;
            }
            catch (e) {
                return "The additional[" + idx + "] value could not be parsed using JSON.stringify().";
            }
        });
    };
    return NGXLoggerUtils;
}());
var LogPosition = /** @class */ (function () {
    function LogPosition(fileName, lineNumber, columnNumber) {
        this.fileName = fileName;
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber;
    }
    LogPosition.prototype.toString = function () {
        return this.fileName + ':' + this.lineNumber + ':' + this.columnNumber;
    };
    return LogPosition;
}());
var NGXMapperService = /** @class */ (function () {
    function NGXMapperService(httpBackend) {
        this.httpBackend = httpBackend;
        this.cache = {};
        this.errorCache = {};
    }
    NGXMapperService.getStackLine = function () {
        var error = new Error();
        return error.stack.split('\n')[5];
    };
    NGXMapperService.getPosition = function (stackLine) {
        var position = stackLine.substring(stackLine.lastIndexOf('\/') + 1, stackLine.indexOf(')'));
        var dataArray = position.split(':');
        if (dataArray.length === 3) {
            return new LogPosition(dataArray[0], +dataArray[1], +dataArray[2]);
        }
        return new LogPosition('unknown', 0, 0);
    };
    NGXMapperService.getTranspileLocation = function (stackLine) {
        return stackLine.substring(stackLine.indexOf('(') + 1, stackLine.indexOf(')'));
    };
    NGXMapperService.getMapFilePath = function (stackLine) {
        var file = NGXMapperService.getTranspileLocation(stackLine);
        var mapFullPath = file.substring(0, file.lastIndexOf(':'));
        return mapFullPath.substring(0, mapFullPath.lastIndexOf(':')) + '.map';
    };
    NGXMapperService.getMapping = function (sourceMap, position) {
        var sourceFileIndex = 0;
        var sourceCodeLine = 0;
        var sourceCodeColumn = 0;
        var lines = sourceMap.mappings.split(';');
        for (var lineIndex = 0; lineIndex < lines.length; lineIndex++) {
            var generatedCodeColumn = 0;
            var columns = lines[lineIndex].split(',');
            for (var columnIndex = 0; columnIndex < columns.length; columnIndex++) {
                var decodedSection = decode(columns[columnIndex]);
                if (decodedSection.length >= 4) {
                    generatedCodeColumn += decodedSection[0];
                    sourceFileIndex += decodedSection[1];
                    sourceCodeLine += decodedSection[2];
                    sourceCodeColumn += decodedSection[3];
                }
                if (lineIndex === position.lineNumber) {
                    if (generatedCodeColumn === position.columnNumber) {
                        return new LogPosition(sourceMap.sources[sourceFileIndex], sourceCodeLine - 1, sourceCodeColumn);
                    }
                    else if (columnIndex + 1 === columns.length) {
                        return new LogPosition(sourceMap.sources[sourceFileIndex], sourceCodeLine - 1, 0);
                    }
                }
            }
        }
        return new LogPosition('unknown', 0, 0);
    };
    NGXMapperService.prototype._getSourceMap = function (sourceMapLocation, distPosition) {
        var _this = this;
        var req = new HttpRequest('GET', sourceMapLocation);
        return this.httpBackend.handle(req).pipe(filter(function (e) { return (e instanceof HttpResponse); }), map(function (httpResponse) { return httpResponse.body; }), map(function (sourceMap) {
            if (!_this.cache.hasOwnProperty(sourceMapLocation)) {
                _this.cache[sourceMapLocation] = sourceMap;
            }
            return NGXMapperService.getMapping(sourceMap, distPosition);
        }), retry(3), catchError(function () {
            _this.errorCache[sourceMapLocation] = true;
            return of(distPosition);
        }));
    };
    NGXMapperService.prototype.getCallerDetails = function (sourceMapsEnabled) {
        var _this = this;
        var stackLine = NGXMapperService.getStackLine();
        return of([
            NGXMapperService.getPosition(stackLine),
            NGXMapperService.getMapFilePath(stackLine)
        ]).pipe(switchMap(function (_a) {
            var _b = __read(_a, 2), distPosition = _b[0], sourceMapLocation = _b[1];
            if (!sourceMapsEnabled || _this.errorCache.hasOwnProperty(sourceMapLocation)) {
                return of(distPosition);
            }
            if (_this.cache.hasOwnProperty(sourceMapLocation)) {
                return of(NGXMapperService.getMapping(_this.cache[sourceMapLocation], distPosition));
            }
            return _this._getSourceMap(sourceMapLocation, distPosition);
        }));
    };
    return NGXMapperService;
}());
NGXMapperService.decorators = [
    { type: Injectable },
];
NGXMapperService.ctorParameters = function () { return [
    { type: HttpBackend }
]; };
var Levels = [
    'TRACE',
    'DEBUG',
    'INFO',
    'LOG',
    'WARN',
    'ERROR',
    'FATAL',
    'OFF'
];
var NGXLogger = /** @class */ (function () {
    function NGXLogger(mapperService, httpService, loggerConfig, platformId) {
        this.mapperService = mapperService;
        this.httpService = httpService;
        this.platformId = platformId;
        this._isIE = isPlatformBrowser(platformId) &&
            !!(navigator.userAgent.indexOf('MSIE') !== -1 || navigator.userAgent.match(/Trident\//) || navigator.userAgent.match(/Edge\//));
        this.config = new NGXLoggerConfigEngine(loggerConfig);
        this._logFunc = this._isIE ? this._logIE.bind(this) : this._logModern.bind(this);
    }
    NGXLogger.prototype.trace = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.TRACE, message, additional);
    };
    NGXLogger.prototype.debug = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.DEBUG, message, additional);
    };
    NGXLogger.prototype.info = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.INFO, message, additional);
    };
    NGXLogger.prototype.log = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.LOG, message, additional);
    };
    NGXLogger.prototype.warn = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.WARN, message, additional);
    };
    NGXLogger.prototype.error = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.ERROR, message, additional);
    };
    NGXLogger.prototype.fatal = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
        this._log(NgxLoggerLevel.FATAL, message, additional);
    };
    NGXLogger.prototype.setCustomHttpHeaders = function (headers) {
        this._customHttpHeaders = headers;
    };
    NGXLogger.prototype.setCustomParams = function (params) {
        this._customParams = params;
    };
    NGXLogger.prototype.registerMonitor = function (monitor) {
        this._loggerMonitor = monitor;
    };
    NGXLogger.prototype.updateConfig = function (config) {
        this.config.updateConfig(config);
    };
    NGXLogger.prototype.getConfigSnapshot = function () {
        return this.config.getConfig();
    };
    NGXLogger.prototype._logIE = function (level, metaString, message, additional) {
        additional = additional || [];
        switch (level) {
            case NgxLoggerLevel.WARN:
                console.warn.apply(console, __spread([metaString + " ", message], additional));
                break;
            case NgxLoggerLevel.ERROR:
            case NgxLoggerLevel.FATAL:
                console.error.apply(console, __spread([metaString + " ", message], additional));
                break;
            case NgxLoggerLevel.INFO:
                console.info.apply(console, __spread([metaString + " ", message], additional));
                break;
            default:
                console.log.apply(console, __spread([metaString + " ", message], additional));
        }
    };
    NGXLogger.prototype._logModern = function (level, metaString, message, additional) {
        var color = NGXLoggerUtils.getColor(level);
        additional = additional || [];
        switch (level) {
            case NgxLoggerLevel.WARN:
                console.warn.apply(console, __spread(["%c" + metaString, "color:" + color, message], additional));
                break;
            case NgxLoggerLevel.ERROR:
            case NgxLoggerLevel.FATAL:
                console.error.apply(console, __spread(["%c" + metaString, "color:" + color, message], additional));
                break;
            case NgxLoggerLevel.INFO:
                console.info.apply(console, __spread(["%c" + metaString, "color:" + color, message], additional));
                break;
            default:
                console.log.apply(console, __spread(["%c" + metaString, "color:" + color, message], additional));
        }
    };
    NGXLogger.prototype._log = function (level, message, additional, logOnServer) {
        var _this = this;
        if (additional === void 0) { additional = []; }
        if (logOnServer === void 0) { logOnServer = true; }
        var config = this.config.getConfig();
        var isLog2Server = logOnServer && config.serverLoggingUrl && level >= config.serverLogLevel;
        var isLogLevelEnabled = level >= config.level;
        if (!(message && (isLog2Server || isLogLevelEnabled))) {
            return;
        }
        var logLevelString = Levels[level];
        message = NGXLoggerUtils.prepareMessage(message);
        var validatedAdditionalParameters = NGXLoggerUtils.prepareAdditionalParameters(additional);
        var timestamp = new Date().toISOString();
        this.mapperService.getCallerDetails(config.enableSourceMaps).subscribe(function (callerDetails) {
            var logObject = {
                message: message,
                additional: validatedAdditionalParameters,
                level: level,
                timestamp: timestamp,
                fileName: callerDetails.fileName,
                lineNumber: callerDetails.lineNumber.toString()
            };
            if (_this._loggerMonitor && isLogLevelEnabled) {
                _this._loggerMonitor.onLog(logObject);
            }
            if (isLog2Server) {
                message = message instanceof Error ? message.stack : message;
                logObject.message = message;
                var headers = _this._customHttpHeaders || new HttpHeaders();
                headers.set('Content-Type', 'application/json');
                var options = {
                    headers: headers,
                    params: _this._customParams || new HttpParams(),
                    responseType: config.httpResponseType || 'json'
                };
                _this.httpService.logOnServer(config.serverLoggingUrl, logObject, options).subscribe(function (res) {
                }, function (error) {
                    _this._log(NgxLoggerLevel.ERROR, "FAILED TO LOG ON SERVER: " + message, [error], false);
                });
            }
            if (isLogLevelEnabled && !config.disableConsoleLogging) {
                var metaString = NGXLoggerUtils.prepareMetaString(timestamp, logLevelString, callerDetails.fileName, callerDetails.lineNumber.toString());
                return _this._logFunc(level, metaString, message, additional);
            }
        });
    };
    return NGXLogger;
}());
NGXLogger.decorators = [
    { type: Injectable },
];
NGXLogger.ctorParameters = function () { return [
    { type: NGXMapperService },
    { type: NGXLoggerHttpService },
    { type: LoggerConfig },
    { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
]; };
var CustomNGXLoggerService = /** @class */ (function () {
    function CustomNGXLoggerService(mapperService, httpService, platformId) {
        this.mapperService = mapperService;
        this.httpService = httpService;
        this.platformId = platformId;
    }
    CustomNGXLoggerService.prototype.create = function (config, httpService, logMonitor, mapperService) {
        var logger = new NGXLogger(mapperService || this.mapperService, httpService || this.httpService, config, this.platformId);
        if (logMonitor) {
            logger.registerMonitor(logMonitor);
        }
        return logger;
    };
    return CustomNGXLoggerService;
}());
CustomNGXLoggerService.decorators = [
    { type: Injectable },
];
CustomNGXLoggerService.ctorParameters = function () { return [
    { type: NGXMapperService },
    { type: NGXLoggerHttpService },
    { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
]; };
var NGXLoggerMonitor = /** @class */ (function () {
    function NGXLoggerMonitor() {
    }
    return NGXLoggerMonitor;
}());
var NGXLogInterface = /** @class */ (function () {
    function NGXLogInterface() {
    }
    return NGXLogInterface;
}());
var LoggerModule = /** @class */ (function () {
    function LoggerModule() {
    }
    LoggerModule.forRoot = function (config) {
        return {
            ngModule: LoggerModule,
            providers: [
                { provide: LoggerConfig, useValue: config || {} },
                NGXLogger,
                NGXLoggerHttpService,
                CustomNGXLoggerService,
                NGXMapperService
            ]
        };
    };
    LoggerModule.forChild = function () {
        return {
            ngModule: LoggerModule,
            providers: [
                NGXLogger,
                NGXLoggerHttpService,
                CustomNGXLoggerService,
                NGXMapperService
            ]
        };
    };
    return LoggerModule;
}());
LoggerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                providers: [
                    NGXLogger,
                    NGXLoggerHttpService,
                    CustomNGXLoggerService,
                    NGXMapperService
                ]
            },] },
];
var NGXLoggerMock = /** @class */ (function () {
    function NGXLoggerMock() {
    }
    NGXLoggerMock.prototype.trace = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.debug = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.info = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.log = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.warn = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.error = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.fatal = function (message) {
        var additional = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            additional[_i - 1] = arguments[_i];
        }
    };
    NGXLoggerMock.prototype.updateConfig = function (config) {
    };
    NGXLoggerMock.prototype.setCustomHttpHeaders = function (headers) {
    };
    NGXLoggerMock.prototype.setCustomParams = function (params) {
    };
    NGXLoggerMock.prototype.registerMonitor = function (monitor) {
    };
    NGXLoggerMock.prototype.getConfigSnapshot = function () {
        return new LoggerConfig();
    };
    return NGXLoggerMock;
}());
var CustomNGXLoggerServiceMock = /** @class */ (function () {
    function CustomNGXLoggerServiceMock() {
    }
    CustomNGXLoggerServiceMock.prototype.create = function () {
        return new NGXLoggerMock();
    };
    return CustomNGXLoggerServiceMock;
}());
var NGXLoggerHttpServiceMock = /** @class */ (function () {
    function NGXLoggerHttpServiceMock() {
    }
    NGXLoggerHttpServiceMock.prototype.logOnServer = function (url, message, additional, timestamp, logLevel) {
        return of({});
    };
    return NGXLoggerHttpServiceMock;
}());
var NGXMapperServiceMock = /** @class */ (function () {
    function NGXMapperServiceMock() {
    }
    NGXMapperServiceMock.prototype.getCallerDetails = function () {
        return of({ fileName: 'test.ts', lineNumber: 0, columnNumber: 0 });
    };
    return NGXMapperServiceMock;
}());
var LoggerTestingModule = /** @class */ (function () {
    function LoggerTestingModule() {
    }
    return LoggerTestingModule;
}());
LoggerTestingModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    { provide: NGXLogger, useClass: NGXLoggerMock },
                    { provide: NGXLoggerHttpService, useClass: NGXLoggerHttpServiceMock },
                    { provide: CustomNGXLoggerService, useClass: CustomNGXLoggerServiceMock },
                    { provide: NGXMapperService, useClass: NGXMapperServiceMock }
                ]
            },] },
];

export { LoggerModule, Levels, NGXLogger, LoggerConfig, CustomNGXLoggerService, NGXLoggerMonitor, NGXLoggerHttpService, NGXMapperService, NGXLoggerUtils, NgxLoggerLevel, NGXLogInterface, NGXLoggerMock, CustomNGXLoggerServiceMock, NGXLoggerHttpServiceMock, LoggerTestingModule };
//# sourceMappingURL=ngx-logger.js.map
